@@ -18,7 +18,9 @@ class DialogNewlyInstalled(SimpleGladeApp):
                                 path=datadir+"/gnome-app-install.glade",
                                 root="dialog_newly_installed",
                                 domain="gnome-app-install")
-                                
+        # we do not support launching apps when runing as root
+        if os.getuid() == 0:
+            self.label_newly_installed.hide()
         self.store = gtk.ListStore(gobject.TYPE_STRING,
                                    gobject.TYPE_PYOBJECT)
         for elm in to_add:
@@ -36,6 +38,8 @@ class DialogNewlyInstalled(SimpleGladeApp):
     def init_treeview(self, view):
         def package_view_func(cell_layout, renderer, model, iter):
             (name, item) = model.get(iter, COL_NAME, COL_ITEM)
+            if not item.menupath:
+                return
             menupath = "</b> &gt; <b>".join(item.menupath)
             menupath = "<b>" + menupath + "</b>"
             renderer.set_property("markup",
