@@ -7,9 +7,13 @@ from gettext import gettext as _
 import gettext
 from BrowserView import GtkHtml2BrowserView as BrowserView
 
+
+# FIXME: this should be renamed to something like
+#        DialogComponentEnable()
+# 
 class DialogUnavailable(SimpleGladeApp):
 
-    def __init__(self, datadir, parent, item):
+    def __init__(self, datadir, parent, item, need_internet=True):
         SimpleGladeApp.__init__(self,
                                 path=datadir+"/gnome-app-install.glade",
                                 root="dialog_unavailable",
@@ -36,7 +40,8 @@ class DialogUnavailable(SimpleGladeApp):
             header = _("Install software from %s?") % item.channel
             msg = _("%s is provided by a third party vendor. The third party vendor "
                     "is responsible for support and security updates.") % item.name
-        msg += "\n\n%s" % _("You need a working internet connection to continue.")
+        if need_internet:
+            msg += "\n\n%s" % _("You need a working internet connection to continue.")
         self.button_unavailable_add.set_label(_("_Install"))
         self.dialog_unavailable.set_transient_for(parent)
         self.dialog_unavailable.realize()
