@@ -1,4 +1,10 @@
-# (c) 2005 Canonical, GPL
+# (c) 2005-2007 Canonical - GPL
+# (c) 2006-2007 Sebastian Heinlein
+#
+# Authors:
+#  Michael Vogt
+#  Sebastian Heinlein
+#
 
 import pygtk
 pygtk.require("2.0")
@@ -31,7 +37,15 @@ from CoreMenu import *
 PIMP_APPS = ["gstreamer0.10-plugins-ugly",
              "gstreamer0.10-ffmpeg",
              "sun-java5-plugin",
-             "flashplugin-nonfree"]
+             "flashplugin-nonfree",
+             "ubuntu-restricted-extras"]
+
+class NullActivationStyleForMenu:
+    # See class ActivationStyle in gnome-app-install; that class
+    # would do just fine here except that it's upside-down from a
+    # layering POV.
+    def __init__(self): self.selectFilter = None
+    def isApproved(self, component, pkgname): return True
 
 class ApplicationMenu(CoreApplicationMenu):
     """ this represents the application menu, the interessting bits are:
@@ -43,13 +57,15 @@ class ApplicationMenu(CoreApplicationMenu):
     debug = 0
 
     def __init__(self, datadir, cachedir, cache, treeview_categories,
-                 treeview_packages, progress, filter=SHOW_ONLY_MAIN,
-                 dontPopulate=False):
+                 treeview_packages, progress,
+		 filter=SHOW_ONLY_MAIN, dontPopulate=False,
+		 activation_style=NullActivationStyleForMenu()):
         self.menudir = datadir+"/desktop"
         self.cache = cache
         self.treeview_categories = treeview_categories
         self.treeview_packages = treeview_packages
-        self.popcon_max = 1
+        self.popcon_max = 10
+        self.activationStyle = activation_style
         
         # cache
         self.pickle = {}
@@ -65,7 +81,6 @@ class ApplicationMenu(CoreApplicationMenu):
 
         # search
         self.searchTerms = []
-        self.mimeSearch = None
         
         # properties for the view
         self.filter = filter
@@ -87,11 +102,18 @@ class ApplicationMenu(CoreApplicationMenu):
         # populate the tree
         # use cached self.pickle (should be renamed to self.categories)
         # and cache self.pkgs_to_app
-        if os.path.exists("%s/menu.p" % cachedir):
+        cacheLoaded = False
+        cname = activation_style.menuCacheName()
+        if cachedir is not None and os.path.exists("%s/%s" % (cachedir,cname)):
             progress.label_action.set_label(_("Loading cache..."))
             import cPickle
-            self.pickle = cPickle.load(open("%s/menu.p" % cachedir))
-        else:
+            try:
+                self.pickle = cPickle.load(open("%s/%s" % (cachedir,cname)))
+                cacheLoaded = True
+            except:
+                pass
+
+        if not cacheLoaded:
             progress.label_action.set_label(_("Collecting application data..."))
             self.desktopEntriesSeen.clear()
             menu = xdg.Menu.parse(os.path.join(self.menudir, "applications.menu"))
@@ -155,7 +177,7 @@ class ApplicationMenu(CoreApplicationMenu):
         item = model.get_value(iter, COL_ITEM)
         if item:
             # check for the various view settings
-            if self.mimeSearch and not self.mimeSearch.approved(
+            if not self.activationStyle.isApproved(
                item.component, item.pkgname):
                 return False
             if self.filter == SHOW_ONLY_MAIN and item.component != "main":
@@ -169,7 +191,7 @@ class ApplicationMenu(CoreApplicationMenu):
             if self.filter == SHOW_ONLY_THIRD_PARTY and item.thirdparty != True:
                 return False
             if self.filter == SHOW_ONLY_INSTALLED and not \
-               item.isInstalled:
+               self.itemIsInstalled(item):
                 return False
             # if we search, do the ranking updates 
             if len(self.searchTerms) > 0:
@@ -188,8 +210,8 @@ class ApplicationMenu(CoreApplicationMenu):
         rank = 100 * item.popcon / self.popcon_max
 
         # special case the mime-search
-        if self.mimeSearch:
-            if self._mimeFilter(item):
+        if self.activationStyle.selectFilter:
+            if self._activationStyleFilter(item):
                 return rank
             else:
                 return None
@@ -213,6 +235,10 @@ class ApplicationMenu(CoreApplicationMenu):
                 rank += 25
                 trigger += " mime"
                 hit = True
+            if self._codecMatch(item, term, fuzzy=True):
+                rank += 15
+                trigger += " codec"
+                hit = True
             if term in item.description.lower():
                 rank += 15
                 trigger += " desc"
@@ -230,21 +256,23 @@ class ApplicationMenu(CoreApplicationMenu):
         return rank
 
     def _mimeMatch(self, item, term, fuzzy=False):
-        for re_pattern in item.mime:
-            # mvo: we get a list of regexp from
-            # pyxdg.DesktopEntry.getMimeType, but it does not
-            # use any special pattern at all, so we use the plain
-            # pattern (e.g. text/html, audio/mp3 here)
-            pattern = re_pattern.pattern
+        for pattern in item.mime:
             if fuzzy and term in pattern:
                 return True
-            elif not fuzzy and re_pattern.match(term):
+            elif not fuzzy and pattern == term:
                 return True
         return False
 
-    def _mimeFilter(self, item):
-        for mime_type in self.searchTerms:
-            if self._mimeMatch(item, mime_type):
+    def _codecMatch(self, item, term, fuzzy=False):
+	for codec in item.codecs:
+	    if fuzzy and term in codec: return True
+	    elif not fuzzy and term == codec: return True
+	return False
+
+    def _activationStyleFilter(self, item):
+	filter = self.activationStyle.selectFilter(self)
+        for term in self.searchTerms:
+            if filter(item, term):
                 return True
         return False
 
@@ -265,9 +293,16 @@ class ApplicationMenu(CoreApplicationMenu):
                     res.add(item)
         return res
 
+    def refreshAfterCacheChange(self, progress):
+        #print "refreshAfterCacheChange"
+        # FIXME: progress information here?
+        for cat in self.pickle:
+            for item in self.pickle[cat]:
+                item.toInstall = (self.cache.has_key(item.pkgname) and
+                                  self.cache[item.pkgname].isInstalled)
+
     def refresh(self, progress):        
         self.real_categories_store.clear()
-
         progress.subOp = _("Loading applications...")
         progress.update(0)
         # add "All" category
@@ -293,11 +328,11 @@ class ApplicationMenu(CoreApplicationMenu):
             progress.update(i/float(lenx)*100.0)
             for item in self.pickle[category]:
                 # get name/description again to make sure that i18n is honored
-                item.name = xmlescape(item.desktop_entry.getName())
-                item.description = xmlescape(item.desktop_entry.getComment())
+                item.name = item.desktop_entry.getName()
+                item.description = item.desktop_entry.getComment()
                 # kde uses generic name *grumpf*
                 if not item.description:
-                    item.description = xmlescape(item.desktop_entry.get('GenericName'))
+                    item.description = item.desktop_entry.get('GenericName')
                 item.icontheme = self.icons
 
                 # add to category
@@ -319,15 +354,17 @@ class ApplicationMenu(CoreApplicationMenu):
                         self.pkg_to_app[pkgname].append(item)
                 else:
                     self.pkg_to_app[pkgname] = [item]
+        # now update the cache dependant part
+        self.refreshAfterCacheChange(progress)
 
-                # update the installed/available information
-                if self.cache.has_key(pkgname):
-                    item.available = True
-                    item.isInstalled = self.cache[pkgname].isInstalled
-                else:
-                    item.available = False
-                    item.isInstalled = False
-                item.toInstall = item.isInstalled
+    def itemAvailable(self, item):
+        """ returns True if the item is available in the current
+            apt cache """
+        return self.cache.has_key(item.pkgname)
+    def itemIsInstalled(self, item):
+        """ returns True if the item is currently installed """
+        return (self.cache.has_key(item.pkgname) and
+                self.cache[item.pkgname].isInstalled)
 
     def initListStores(self, category, parent):
         category.icontheme = parent.icons
@@ -349,9 +386,9 @@ class ApplicationMenu(CoreApplicationMenu):
         to_rm = set()
         for (name, item) in self.store:
             for (name,item,popcon) in item.all_applications:
-                if item.isInstalled and not item.toInstall:
+                if self.itemIsInstalled(item) and not item.toInstall:
                     to_rm.add(item)
-                if not item.isInstalled and item.toInstall:
+                if not self.itemIsInstalled(item) and item.toInstall:
                     to_inst.add(item)
         #print "to_add: %s" % to_inst
         #print "to_rm: %s" % to_rm
@@ -361,7 +398,7 @@ class ApplicationMenu(CoreApplicationMenu):
         """ check if there are changes at all """
         for (cat_name, cat)  in self.store:
             for (name,item,popcon) in cat.all_applications:
-                if item.toInstall != item.isInstalled:
+                if item.toInstall != self.itemIsInstalled(item):
                     return True
         return False
 
