@@ -1,8 +1,9 @@
-# (c) 2005 Canonical, GPL
+# (c) 2005-2007 Canonical, GPL
 
 import apt_pkg
 import subprocess
 import gtk
+import gtk.gdk
 import thread
 import time
 import os
@@ -16,27 +17,36 @@ class PackageWorker:
 
     # synaptic actions
     (INSTALL, UPDATE) = range(2)
+
+    def __init__(self, addon_cd=None):
+        self.addon_cd=addon_cd
     
     def run_synaptic(self, id, lock, to_add=None,to_rm=None, action=INSTALL):
         #apt_pkg.PkgSystemUnLock()
         #print "run_synaptic(%s,%s,%s)" % (id, lock, selections)
-        cmd = ["/usr/bin/gksu",
-               "--desktop", "/usr/share/applications/synaptic.desktop",
-               "--",
-               "/usr/sbin/synaptic",
-               "--hide-main-window",
-               "--non-interactive",
-               "--parent-window-id", "%s" % (id) ]
+        cmd = []
+        if os.getuid() != 0:
+            cmd = ["/usr/bin/gksu",
+                   "--desktop", "/usr/share/applications/synaptic.desktop",
+                   "--"]
+        cmd += ["/usr/sbin/synaptic",
+                "--hide-main-window",
+                "--non-interactive",
+                "--parent-window-id", "%s" % (id) ]
 
         # create tempfile for install (here because it must survive
         # durng the synaptic call
         f = tempfile.NamedTemporaryFile()
         if action == self.INSTALL:
+            # setup the cdrom 
+            if self.addon_cd:
+                cmd += ["-o","Acquire::cdrom::mount=%s" % self.addon_cd]
+            # install the stuff
             for item in to_add:
-                f.write("%s\tinstall\n" % item.pkgname)
+                f.write("%s\tinstall\n" % item)
                 #print item.pkgname
             for item in to_rm:
-                f.write("%s\tuninstall\n" % item.pkgname)
+                f.write("%s\tuninstall\n" % item)
             cmd.append("--set-selections-file")
             cmd.append("%s" % f.name)
             f.flush()
@@ -86,10 +96,9 @@ class PackageWorker:
         
         return win, socket
     
-    def perform_action(self, window_main, to_add=None, to_rm=None, action=INSTALL):
+    def perform_action(self, window_main, cache, to_add=None, to_rm=None, action=INSTALL):
         window_main.set_sensitive(False)
-        #plug_win, socket = self.get_plugged_win(window_main)
-        
+        window_main.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
         lock = thread.allocate_lock()
         lock.acquire()
         t = thread.start_new_thread(self.run_synaptic,(window_main.window.xid,lock,to_add, to_rm, action))
@@ -97,6 +106,6 @@ class PackageWorker:
             while gtk.events_pending():
                 gtk.main_iteration()
             time.sleep(0.05)
-        #plug_win.destroy()
         window_main.set_sensitive(True)
+        window_main.window.set_cursor(None)
         return self.return_code
