@@ -1,5 +1,10 @@
 #! /usr/bin/env python
 #
+# (c) 2005-2007 Canonical, GPL
+# Authors:
+#  Michael Vogt
+#
+#
 # FIXME: strip "TryExec" from the extracted menu files (and noDisplay)
 #        
 # TODO:
@@ -24,6 +29,8 @@ import string
 import shutil
 import urllib
 import logging
+import fnmatch
+import glob
 
 
 # these globals here suck
@@ -31,6 +38,8 @@ tmpdir =  tempfile.mkdtemp()
 blacklist = "blacklist.cfg"
 blacklist_desktop = "blacklist_desktop.cfg"
 renamecfg = "rename.cfg"
+annotatecfg = "annotate.cfg"
+codecs_foradditional = { }
 
 # packages we have already seen 
 pkgs_seen = set()
@@ -45,6 +54,9 @@ desktop_blacklist = set()
 # (e.g.abiword-plugins -> abiword-gnome)
 pkgs_transform = {}
 
+# a mapping from desktop file to a list of free annotations that will
+# be added to the desktop file (useful for e.g. X-AppInstall-Replaces
+desktop_annotate = {}
 
 # a dictionary with "$arch" -> set() of packages mapping
 # this makes it easy to analyse what packages are only
@@ -57,7 +69,7 @@ pkgs_to_desktop = {}
 
 # all supported arches
 SUPPORTED_ARCHES = ("i386","amd64","powerpc")
-
+#SUPPORTED_ARCHES = ("i386")
 
 def getMemberFromAr(arFile, member):
   """ helper to extract the files """
@@ -119,7 +131,8 @@ def search_icon(tarfile, iconName, outputdir):
                  "usr/share/pixmaps",
                  "usr/share/icons/hicolor/32x32",
                  "usr/share/icons/hicolor/22x22",
-                 "usr/share/icons/hicolor/16x16"
+                 "usr/share/icons/hicolor/16x16",
+		 "usr/share/icons"
                  ]
   # extensions (ordered by importance)
   pixmaps_ext = ["", ".png",".xpm",".svg"]
@@ -135,6 +148,14 @@ def search_icon(tarfile, iconName, outputdir):
   return (False, None)
 
 
+def tarfile_extract_orlog(dataFile, path):
+  try:
+    file = dataFile.extractfile(path)
+  except KeyError, e:
+    logging.error("Failed to extract '%s' from '%s' (%s)" %
+                     (path, dataFile.name, e))
+    return None
+  return file
 
 def getFiles(dataPath, pkgname, section, outputdir):
   """ actually extract the files from a data.tar.gz of a package """
@@ -162,16 +183,26 @@ def getFiles(dataPath, pkgname, section, outputdir):
     logging.error("Choked on tarball. Package: '%s'" % package)
     return # too many bad debs. take this line out if you fix that
 
+  # and look for any codec support list
+  codecs = set()
+  for filename in dataFile.getnames():
+    if not (fnmatch.fnmatch(filename, 'usr/share/gstreamer-*/plugin-info/*.supported') or fnmatch.fnmatch(filename, './usr/share/gstreamer-*/plugin-info/*.supported')): continue
+    codecsfile = tarfile_extract_orlog(dataFile,filename);
+    if not codecsfile:
+      continue
+    logging.debug("reading codec list file %s" % filename)
+    for line in codecsfile.readlines():
+      line = string.strip(line)
+      if ';' in line:
+        logging.warning("Codec string '%s' from '%s' in '%s' contains ';'" %
+                        (line, filename, dataFile.name))
+        continue
+      codecs.add(line)
 
   # look at the desktop files
   for path in desktopPaths:
-    try:
-      desktopfile = dataFile.extractfile(path)
-    except KeyError, e:
-      logging.error("Failed to extract '%s' from '%s' (%s)" % (path,
-                                                               dataFile.name,
-                                                               e))
-      continue
+    desktopfile = tarfile_extract_orlog(dataFile, path)
+    if not desktopfile: continue
     # new location for the desktopfile
     desktop = os.path.join(outputdir, os.path.basename(path))
     # extract the icon
@@ -194,8 +225,7 @@ def getFiles(dataPath, pkgname, section, outputdir):
         pkgname = parentpkg
 
     # now write out the file
-    logging.debug("Wrote desktop file '%s' for arch '%s'" % (desktopfile.name,
-                                                             arch))
+    logging.debug("Writing desktop file '%s' for arch '%s'" % (desktopfile.name, arch))
     outfile = open(desktop, "w")
     desktopfile.seek(0)
     for line in desktopfile.readlines():
@@ -206,12 +236,24 @@ def getFiles(dataPath, pkgname, section, outputdir):
       outfile.write(line)
     outfile.write("X-AppInstall-Package=%s\n" % pkgname)
     outfile.write("X-AppInstall-Section=%s\n" % section)
+    if codecs:
+      outfile.write("X-AppInstall-Codecs=%s\n" % ';'.join(codecs))
+    desktopfile_name = os.path.basename(desktopfile.name)
+    if desktop_annotate.has_key(desktopfile_name):
+      logging.info("adding annontate '%s' for '%s'" % (desktop_annotate[desktopfile_name], desktopfile_name))
+      for a in desktop_annotate[desktopfile_name]:
+        outfile.write("%s\n" % a.strip())
     outfile.close()
-    pkgs_to_desktop[pkgname] = os.path.basename(outfile.name)
+    if not pkgs_to_desktop.has_key(pkgname):
+      pkgs_to_desktop[pkgname] = []
+    pkgs_to_desktop[pkgname].append(os.path.basename(outfile.name))
 
     # close the desktop file
     desktopfile.close()
 
+  if codecs:
+    try: codecs_foradditional[pkgname] &= codecs
+    except KeyError: codecs_foradditional[pkgname] = codecs
 
 def processDeb(debPath, pkgname, section,
                outputdir=os.path.join(os.getcwd(), "menu-data")):
@@ -327,9 +369,18 @@ if __name__ == "__main__":
         (oldname,newname) = string.split(string.strip(line))
         logging.debug("renaming: %s -> %s" % (oldname,newname))
         pkgs_transform[oldname] = newname
+
+  if os.path.exists(annotatecfg):
+    logging.info("using annotates: '%s'" % annotatecfg)
+    for line in open(annotatecfg):
+      line = line.strip()
+      if line != "" and not line.startswith("#"):
+        (desktopfile,annotations_str) = string.split(line,":")
+        annotations = annotations_str.split(",")
+        logging.debug("annotations: '%s': %s" % (desktopfile,annotations))
+        desktop_annotate[desktopfile] = annotations
     
   for arch in SUPPORTED_ARCHES:
-
     # fake a $arch machine
     # FIXME: we want arch-independent stuff in the long run
     #        e.g. for Mac-On-Linux (MoL)
@@ -385,13 +436,48 @@ if __name__ == "__main__":
       if not pkgs_to_desktop.has_key(pkg):
         logging.debug("Skipping arch-specific pkg '%s' with no desktop file" % pkg)
         continue
-      dfile = "menu-data/"+pkgs_to_desktop[pkg]
-      if os.path.exists(dfile):
-        logging.debug("adding arch-specifc information to %s (%s)" % (dfile,arches))
-        open(dfile,"a").write(str)
-      else:
-        logging.error("can't find desktopfile for: %s (%s)" % (pkg,dfile))
-    
+      for f in pkgs_to_desktop[pkg]:
+        dfile = "menu-data/"+f
+        if os.path.exists(dfile):
+          logging.debug("adding arch-specifc information to %s (%s)" % (dfile,arches))
+          open(dfile,"a").write(str)
+        else:
+          logging.error("can't find desktopfile for: %s (%s)" % (pkg,dfile))
+
+  # edit the codec lists into the entries in menu-data-additional
+  logging.debug("adding codecs: '%s'", codecs_foradditional)
+  seen = set()
+  for filename in glob.glob('menu-data-additional/*.desktop'):
+    lines = []
+    file = open(filename)
+    pkgname = None
+    for line in file:
+      line = string.strip(line)
+      if re.match("^X-AppInstall-Codecs\s*=", line): continue
+      lines.append(line)
+      m = re.match("^X-AppInstall-Package\s*=\s*(\S+)\s*$", line)
+      if m: pkgname = m.group(1)
+    file.close()
+    if not pkgname:
+      logging.warning("file '%s' doesn't specify an X-AppInstall-Package" %
+                      filename)
+      continue
+    try:
+      codecs = codecs_foradditional[pkgname]
+    except KeyError:
+      continue
+    logging.debug("adding codec info to '%s' " % pkgname)
+    lines.append("X-AppInstall-Codecs=%s" % ';'.join(codecs))
+    seen.add(pkgname)
+    new_file = open(filename+'.tmp', 'w');
+    for line in lines: print >>new_file, line
+    new_file.close()
+    os.rename(filename+'.tmp', filename)
+
+  missing = set(codecs_foradditional.keys())-seen
+  if len(missing) > 0:
+    logging.warning("Packages with codecs but no data in menu-data-additonal: %s" % missing)
+      
   # be nice, clean-up
   if tmpdir != None and tmpdir != "":
     shutil.rmtree(tmpdir)
